<?php

App::uses('BuzzBookingsAppController', 'BuzzBookings.Controller');
App::uses('CustomerAddress', 'BuzzCustomers.Model');
App::uses('PurchasableInterface', 'BuzzPurchase.Lib/Interface');

class BookingsController extends BuzzBookingsAppController implements PurchasableInterface {

/**
 * List of admin actions the controller supports
 *
 * @var array
 */
	public $adminActions = array(
		'admin_index',
		'admin_edit',
		'admin_delete',
		'admin_email_confirmation'
	);

/**
 * Components
 *
 * @var array
 */
	public $components = array(
		'BuzzPurchase.Purchase',
		'Transactions.Transactions'
	);

/**
 * Helpers
 */
	public $helpers = array(
		'Address.Address'
	);

	public function beforeFilter() {
		parent::beforeFilter();

		$this->Auth->allow(array(
			'ajax_dates',
			'ajax_remove_voucher',
			'ajax_times',
			'ajax_voucher',
			'basket',
			'checkout',
			'confirmation',
			'date',
			'extras',
			'index',
			'packages',
			'payment_callback',
			'quit',
			'restore',
			'timeout'
		));
	}

/**
 * Bookings landing page - will mostly get bypassed by homepage/activity pages
 *
 * @return void
 */
	public function index() {
		$Booking = $this->{$this->modelClass};

		if (!empty($this->request->data)) {

			if (!empty($this->request->data['Booking']['activity_id'])) {
				// Forward to the correct packages step.
				return $this->redirect([
					'action' => 'packages',
					$this->request->data['Booking']['activity_id']
				]);
			} else {
				$this->Session->setFlash(
					__d('buzz_bookings', 'Please select an activity'),
					'flash_fail'
				);
			}

		}

		$activities = $Booking->Activity->getListed();
		$activities = Hash::combine($activities, '{n}.Activity.id', '{n}.Activity.name');

		if (count($activities) === 1) {
			return $this->redirect([
				'action' => 'packages'
			]);
		} elseif (empty($activities)) {
			// No activities have been setup or created so throw an exception
			throw new NotFoundException();
		}

		$this->set(compact('activities'));
		$this->view = 'BuzzBookings./Bookings/index';

		return;
	}

/**
 * Packages step - add packages and vouchers to a booking
 *
 * To jump to the packages step for a specific activity the activity ID can be
 * passed; otherwise the activity ID is set on the booking.
 *
 * To preselect a specific package a package ID needs to be passed.
 *
 * @param int $activityId Activity ID
 * @param int $packageId Optional activity package ID
 * @return void
 */
	public function packages($activityId = null, $packageId = null) {
		$Booking = $this->{$this->modelClass};

		// Process form data.
		if (!empty($this->request->data)) {

			$result = $Booking->createBasket($activityId, $this->request->data['BookingItem']);
			if ($result === true) {
				return $this->redirect(['action' => 'date']);
			} else {
				$this->Session->setFlash(
					__d('buzz_bookings', 'Please select a package'),
					'flash_fail'
				);
			}

		}

		// Get the basket.
		$basket = $Booking->getBasket();

		if (empty($activityId)) {

			if (!empty($basket['Booking']['activity_id'])) {
				// Use the basket's activity.
				$activityId = $basket['Booking']['activity_id'];
			} else {
				// Check if the site is configured to only have one listed
				// bookable activity. This is needed to prevent a redirect
				// loop with this and the previous step.
				$activityId = $Booking->Activity->onlyBookableActivity();
			}

		}

		// If the activity ID hasn't been set then redirect to the previous
		// step.
		if (empty($activityId)) {
			return $this->redirect([
				'action' => 'index'
			]);
		} else {
			// Set the booking's activity ID.
			$this->request->data['Booking']['activity_id'] = $activityId;
		}

		// Get the packages and activity details for generating the form.
		$data = $Booking->Activity->getBookableActivity($activityId);

		if (empty($data)) {
			throw new NotFoundException();
		}

		if (
			!empty($basket) && (int)$basket['Booking']['activity_id'] !== (int)$activityId
		) {
			// Clear the basket as we're either changing the activity being booked so
			// existing items are no longer valid.
			$Booking->clearBasket(false);
			// We don't need to pass the basket to the view anymore as we're
			// essentially starting a fresh basket.
			$basket = [];
		} elseif (empty($this->request->data['BookingItem']) && !empty($basket['BookingItem'])) {
			// Populate the form with existing booking items.
			$this->request->data['BookingItem'] = Hash::combine($basket['BookingItem'], '{n}.activity_package_id', '{n}');
		}

		// If someone's followed a link to directly book a specific package
		// then set the quantity to 1.
		if (!empty($packageId) && empty($this->request->data['BookingItem'])) {
			$this->request->data['BookingItem'][$packageId]['quantity'] = 1;
		}

		$previousStep = ['action' => 'index'];
		// Check if this is a non-listed activity (i.e. we will need to return
		// the user to the activity page rather than allow them to select a
		// different activity).
		if (!empty($basket['Activity']['listed']) && (bool)$basket['Activity']['listed'] === false) {
			$previousStep = ['controller' => 'activities', 'action' => 'view', $activityId];
		}

		// Get the available vouchers.
		$this->_populateVouchers($activityId, $data['Activity']['api_reference'], true);

		$this->set(compact('data', 'basket', 'previousStep'));
		$this->view = 'BuzzBookings./Bookings/packages';

		return;
	}

/**
 * Date step - book a time and date for the booking
 *
 * @return void
 */
	public function date() {
		$Booking = $this->{$this->modelClass};

		// Get the basket.
		$basket = $Booking->getBasket();
		if (empty($basket)) {
			$this->Session->setFlash(
				__d('buzz_bookings', 'Your session has expired'),
				'flash_fail'
			);
			return $this->redirect(['action' => 'index']);
		}

		if ($Booking->clearTemporaryBookings() === false) {
			throw new InternalErrorException();
		}

		if (!empty($this->request->data)) {

			$data = $this->Session->read('BuzzBookings.availableDates.' . $this->request->data['Booking']['date'] . '.' . $this->request->data['Booking']['time']);

			if ($Booking->addTemporaryBooking($basket, $this->request->data['Booking']['date'], $data) === true) {
				// Get rid of the available dates from the session (these will
				// need regenerating if the customer returns to this step).
				$this->Session->delete('BuzzBookings.availableDates');
				// Start the booking timer.
				$this->_startTimer();
				// Redirect to the next step.
				return $this->redirect(['action' => 'extras']);
			} else {
				// The booking failed, so we want to ask the customer to pick a
				// new date/time.
				$this->Session->setFlash(
					__d('buzz_bookings', 'Your selected booking is no longer available, please try again'),
					'flash_fail'
				);
				return $this->redirect(['action' => 'date']);
			}

		}

		$previousStep = ['action' => 'packages', $basket['Booking']['activity_id']];

		$this->set(compact('basket', 'previousStep'));
		$this->view = 'BuzzBookings./Bookings/date';

		return;
	}

	public function extras() {
		$Booking = $this->{$this->modelClass};

		$this->_checkTimeRemaining();

		// Get the basket.
		$basket = $Booking->getBasket();
		if (empty($basket)) {
			$this->Session->setFlash(
				__d('buzz_bookings', 'Your session has expired'),
				'flash_fail'
			);
			return $this->redirect(['action' => 'index']);
		} elseif (empty($basket['Booking']['booking_date'])) {
			return $this->redirect(['action' => 'date']);
		}

		if (!empty($this->request->data)) {
			if ($Booking->addUpsells($this->request->data['BookingExtra'], []) === true) {
				return $this->redirect(['action' => 'basket']);
			}
			$this->Session->setFlash(
				__d('buzz_bookings', 'There was a problem adding your extras to your booking'),
				'flash_fail'
			);
		} else {
			// We need to clear the extras from the basket. They will get
			// re-added when the user proceeds to the next step, but we need
			// to make sure that any bookable items can still be booked before
			// proceeding.
			$Booking->clearExtras();
			// Set form data.
			$this->request->data['BookingExtra'] = Hash::combine($basket['BookingExtra'], '{n}.extra_id', '{n}');
		}

		$extras = $Booking->getExtras($basket);

		$previousStep = ['action' => 'date'];

		$this->set(compact('basket', 'extras', 'previousStep', '_checkTimeRemaining'));
		$this->view = 'BuzzBookings./Bookings/extras';

		return;
	}

	public function basket() {
		$Booking = $this->{$this->modelClass};

		$this->_checkTimeRemaining();

		// Get the basket.
		$basket = $Booking->getBasket();
		if (empty($basket)) {
			$this->Session->setFlash(
				__d('buzz_bookings', 'Your session has expired'),
				'flash_fail'
			);
			return $this->redirect(['action' => 'index']);
		} elseif (empty($basket['Booking']['booking_date'])) {
			return $this->redirect(['action' => 'date']);
		}

		if (!empty($this->request->data)) {
			return $this->redirect(['action' => 'checkout']);
		} else {
			// We set the basket ID for the form just so that there's some data
			// to submit in the form to handle redirecting to the next step.
			// This value should *not* be used directly, always retrieve the
			// basket ID from the session!
			$this->request->data['Booking']['id'] = $Booking->getBasketId();
		}

		$previousStep = ['action' => 'extras'];

		$this->set(compact('basket', 'previousStep'));
		$this->view = 'BuzzBookings.basket';

		return;
	}

/**
 * Allow customers to quit the booking and have booking details emailed to
 * them.
 *
 * @return void
 */
	public function quit() {
		$Booking = $this->{$this->modelClass};

		$this->_checkTimeRemaining();

		// Get the basket.
		$basket = $Booking->getBasket();

		if (!empty($this->request->data)) {
			if ($Booking->fallout($this->request->data['Booking']['email']) === true) {
				return $this->redirect('/');
			} else {
				$this->Session->setFlash(
					__d('buzz_bookings', 'An invalid email was supplied'),
					'flash_fail'
				);
				return $this->redirect(['action' => 'basket']);
			}
		}

		$this->view = 'BuzzBookings.quit';
		$this->layout = 'ajax';

		return;
	}

/**
 * Restores a quit booking from a fallout email
 *
 * @param int $id
 * @param string $signature Hashed email address and booking ID
 * @return void
 */
	public function restore($id, $signature) {
		$Booking = $this->{$this->modelClass};

		$data = $Booking->restoreBooking($id, $signature);

		if (!empty($data)) {
			// The basket has been restored.
			if (empty($data['Booking']['booking_date'])) {
				$this->Session->setFlash(
					__d('buzz_bookings', 'Sorry, your booking is no longer available'),
					'flash_fail'
				);
				return $this->redirect(['action' => 'date']);
			}
			// Start the booking timer.
			$this->_startTimer();
			// Continue to where the customer left off.
			$this->Session->setFlash(
				__d('buzz_bookings', 'Your booking has been restored'),
				'flash_success'
			);
			return $this->redirect(['action' => 'basket']);
		} else {
			$this->Session->setFlash(
				__d('buzz_bookings', 'Sorry, we were unable to find this booking'),
				'flash_fail'
			);
			return $this->redirect(['action' => 'index']);
		}
	}

/**
 * Timeout method for generating a flash message before redirecting to the
 * date step.
 *
 * @return void Redirect to the date step
 */
	public function timeout() {
		$this->Session->setFlash(
			__d('buzz_bookings', 'Your session has timed out'),
			'flash_fail'
		);
		return $this->redirect(['action' => 'date']);
	}

/**
 * Start the booking timer.
 *
 * @return int Seconds remaining
 */
	protected function _startTimer() {
		$allowedBookingTime = Configure::read('BuzzBookings.allowed_booking_time');

		$now = time();
		$data = [
			'start_time' => $now,
			'end_time' => $now + $allowedBookingTime
		];
		$this->Session->write('BuzzBookings.timer', $data);

		return $time;
	}

/**
 * Checks the time remaining for the booking.
 *
 * @return int Seconds remaining
 */
	protected function _checkTimeRemaining() {
		$now = time();
		$data = $this->Session->read('BuzzBookings.timer');

		$timeRemaining = 0;
		if (!empty($data)) {
			$timeRemaining = $data['end_time'] - $now;
		}

		$timeoutRedirect = ['action' => 'timeout'];

		if ($timeRemaining <= 0) {
			return $this->timeout();
		} else {
			$this->set(compact('timeoutRedirect', 'timeRemaining'));
		}

		return $timeRemaining;
	}

/**
 * AJAX call for generating voucher fields.
 *
 * @param int $activityId Activity ID
 * @return void
 */
	public function ajax_voucher($activityId) {
		$Booking = $this->{$this->modelClass};

		$result = false;
		if (!empty($this->request->query['data'])) {
			$result = $Booking->addVoucher($activityId, $data = $this->request->query['data']['BookingItemVoucher']);
			if ($result !== false) {
				$this->set('voucher', $result);

				$this->set(compact('activityId'));
				$this->layout = 'ajax';
				$this->view = 'BuzzBookings.ajax_voucher_added';

				return;
			} else {
				// Set form data.
				$this->request->data = $this->request->query['data'];
				// Invalidate fields
				$Booking->BookingItem->BookingItemVoucher->invalidate('voucher_code');
				$Booking->BookingItem->BookingItemVoucher->invalidate('expiry_date');
			}
		}

		$this->_populateVouchers($activityId);

		$this->set(compact('activityId'));
		$this->layout = 'ajax';
		$this->view = 'BuzzBookings.ajax_voucher';

		return;
	}

/**
 * AJAX call for removing a voucher from a booking.
 *
 * @param int $bookingItemId Booking item record ID
 * @return void
 */
	public function ajax_remove_voucher($bookingItemId) {
		$Booking = $this->{$this->modelClass};

		$this->autoRender = false;

		// Remove the voucher/booking item (this checks it belongs to the user
		// too).
		$result = $Booking->removeVoucher($bookingItemId);

		return json_encode($result);
	}

/**
 * AJAX call for generating the available times and dates for the current
 * booking.
 *
 * @return void
 */
	public function ajax_dates() {
		$Booking = $this->{$this->modelClass};

		// Get the available times and dates.
		$fromDate = date('Y-m-d');
		$toDate = date('Y-m-d', strtotime('+3 months'));
		$data = $Booking->getAvailableDates($fromDate, $toDate);

		if (!empty($data)) {
			// Work out the unavailable dates within our date range.
			$unavailableDates = $Booking->calculateUnavailableDates($data, $fromDate, $toDate);

			// Store the available dates/times in the session.
			$this->Session->write('BuzzBookings.availableDates', Hash::combine($data, '{n}.date', '{n}.times'));

			$this->set(compact('unavailableDates', 'fromDate', 'toDate'));
			$view = 'ajax_dates';
		} else {
			$view = 'ajax_dates_unavailable';
		}

		$this->layout = 'ajax';
		$this->view = 'BuzzBookings./Bookings/' . $view;

		return;
	}

/**
 * AJAX call for displaying available times.
 *
 * @param string $date
 * @return void
 */
	public function ajax_times($date) {
		$Booking = $this->{$this->modelClass};

		$data = $this->Session->read('BuzzBookings.availableDates.' . $date);

		// If no data is returned we don't want to return anything. This is to
		// take into account the date picker loading with the current date and
		// no times being available.
		if (empty($data)) {
			$this->autoRender = false;
			return;
		}

		$this->set(compact('data', 'date'));
		$this->layout = 'ajax';
		$this->view = 'BuzzBookings./Bookings/ajax_times';

		return;
	}

/**
 * Checkout step
 *
 * @return void
 */
	public function checkout() {
		$Booking = $this->{$this->modelClass};

		$this->_checkTimeRemaining();

		// Get the basket.
		$basket = $Booking->getBasket();
		if (empty($basket)) {
			$this->Session->setFlash(
				__d('buzz_bookings', 'Your session has expired'),
				'flash_fail'
			);
			return $this->redirect(['action' => 'index']);
		} elseif (empty($basket['Booking']['booking_date'])) {
			return $this->redirect(['action' => 'date']);
		}

		if (!empty($this->request->data)) {

			$result = $Booking->addBillingDetails($this->request->data['CustomerAddress']);
			if ($result === true) {
				// Take payment.
				$this->_pay();
			} else {
				$this->Session->setFlash(
					__d('buzz_bookings', 'Please correct the errors below'),
					'flash_fail'
				);
			}

		} else {

			// Set the address field defaults (these are configured by the
			// CustomerAddress plugin to make it easier to override for each site).
			$this->request->data['CustomerAddress']['country_id'] = CustomerAddress::getDefaultCountry();
			$this->request->data['CustomerAddress']['us_state_id'] = CustomerAddress::getDefaultUsState();

		}

		// Get the basket.
		$basket = $Booking->getBasket();

		// Get gift voucher purchase conditions
		$this->loadModel('BuzzConditions.Condition');
		$conditions = $this->Condition->getConditions('Booking');

		$previousStep = ['action' => 'basket'];

		$this->set(compact('basket', 'conditions', 'previousStep'));
		$this->set('cardTypes', $this->Purchase->getCardTypes());
		$this->_populateLookups();
		$this->view = 'BuzzBookings.checkout';

		return;
	}

/**
 * Processes the payment and triggers the completion of the payment on success
 *
 * @return void
 */
	protected function _pay() {
		$Booking = $this->{$this->modelClass};

		// Get the latest basket.
		$basket = $Booking->getBasket();

		// If skipPayment is set we bypass payment (this is to making testing
		// simpler and avoid duplicating transaction IDs when testing locally).
		if (Configure::check('skipPayment') === false && $basket['Booking']['total_cost'] > 0) {

			if (Configure::read('BuzzPurchase.onsite') === true) {
				$payment = $this->request->data['Payment'];

				// Split name into first and second names for payment gateways.
				$fullName = trim($payment['card_name']);
				$splitName = explode(' ', $fullName);
				$payment['last_name'] = array_pop($splitName);
				$payment['first_name'] = count($splitName) ? implode(' ', $splitName) : '';
			} else {
				$payment = [];
			}

			$transaction = $this->Transactions->takePayment(
				// Return URLs
				array(
					'return' => Router::url(['action' => 'payment_callback'], true),
					'cancel' => Router::url(['action' => 'checkout'], true)
				),
				// Calling record
				array(
					'model' => 'BuzzBookings.Booking',
					'model_id' => $basket['Booking']['id']
				),
				// Amount to be paid
				$basket['Booking']['total_cost'],
				// Items
				array(),
				// Extras
				array(
					'language' => Configure::read('Config.language'),
					'card' => $payment,
					'address' => $basket['CustomerAddress'],
					'user' => array('email' => $basket['CustomerAddress']['email']),
					'description' => __d(
						'buzz_bookings',
						'%s Booking %d',
						[Configure::read('SiteSetting.site_title'), $basket['Booking']['id']]
					)
				),
				0,
				Configure::read('Transactions.gateway')
			);

			if (!empty($transaction)) {
				// Payment taken successfully, complete purchase.
				return $this->_completePurchase();
			} else {
				$this->Session->setFlash(
					__d(
						'buzz_bookings',
						'There was a problem processing your payment, please check your details and try again'
					),
					'flash_fail'
				);
			}

		} else {
			// Nothing to pay, complete purchase.
			return $this->_completePurchase();
		}


		return;
	}

/**
 * Complete the purchase (after payment received)
 *
 * @return void Redirects to confirmation page
 */
	protected function _completePurchase() {
		$Booking = $this->{$this->modelClass};

		$basketId = $Booking->getBasketId();

		if (empty($basketId)) {
			throw new ForbiddenException();
		}

		$Booking->completePurchase(true);

		// The hash *must* be set after completing the purchase as
		// completePurchase() will delete the session.
		$this->Session->write('Booking.hash', $Booking->hashBasketId($basketId));

		return $this->redirect([
			'action' => 'confirmation',
			$basketId
		]);
	}

/**
 * Confirmation page
 *
 * @param int $id Basket ID
 * @return void
 */
	public function confirmation($id) {
		$Booking = $this->{$this->modelClass};

		if ($Booking->hashBasketId($id) === $this->Session->read('Booking.hash')) {

			$this->Session->delete('Booking.hash');

			$basket = $Booking->getPurchase($id);

			if (Configure::check('BuzzBookings.confirmation_page_id') === true) {
				$this->assignPage(Configure::read('BuzzBookings.confirmation_page_id'));
			}

			$this->set(compact('basket'));
			$this->view = 'BuzzBookings.confirmation';

		} else {

			throw new ForbiddenException();

		}

		return;
	}

/**
 * Populate lookups for address fields.
 *
 * @return void
 */
	protected function _populateLookups() {
		$Booking = $this->{$this->modelClass};

		$countries = ClassRegistry::init('BuzzCustomers.Country')->translatedList();
		$usStates = $Booking->CustomerAddress->UsState->find('list');

		$this->set(compact('countries', 'usStates'));

		return;
	}

/**
 * Populate voucher details
 *
 * @param int $activityId
 * @param int $apiRef
 * @param bool $forceUpdate Pass true to make sure vouchers are fetched from the API not session
 * @return void
 */
	protected function _populateVouchers($activityId, $apiRef = null, $forceUpdate = false) {
		$Booking = $this->{$this->modelClass};

		if ($this->Session->check('BuzzBookings.vouchers') === false || $forceUpdate === true) {

			if (empty($apiRef)) {
				$activity = $Booking->Activity->findById($activityId);
				$apiRef = $activity['Activity']['api_reference'];
			}

			$data = $Booking->getVouchers($apiRef);

		} else {

			$data = $this->Session->read('BuzzBookings.vouchers');

		}

		// Build a simple array of voucher providers for convenience in the View.
		if (!empty($data)) {
			$voucherProviders = Hash::combine(
				$data,
				'{n}.VoucherProvider.id',
				'{n}.VoucherProvider.name'
			);
			asort($voucherProviders);
			$vouchers = $data;
			unset($vouchers[$Booking->getApiVoucherProvider()]);
			$vouchers = Hash::combine(
				$vouchers,
				'{n}.VoucherProvider.Voucher.{n}.id',
				'{n}.VoucherProvider.Voucher.{n}.description',
				'{n}.VoucherProvider.id'
			);
		} else {
			$voucherProviders = [];
			$vouchers = [];
		}

		$this->set(compact('voucherProviders', 'vouchers'));

		return;
	}

/**
 * Generic payment gateway callback.
 *
 * @return void
 */
	public function payment_callback() {
		$result = $this->Transactions->checkPayment();

		if ($result === true) {
			return $this->_completePurchase();
		} else {
			$this->Session->setFlash(
				__d('buzz_bookings', 'There was a problem processing your payment, please try again'),
				'flash_fail'
			);
			return $this->redirect([
				'action' => 'checkout'
			]);
		}
	}

/**
 * Admin index paginate
 *
 * @return array
 */
	protected function _adminIndexPaginate() {
		$conditions = $this->_processFilter();

		$paginate = array(
			'conditions' => $conditions,
			'contain' => array(
				'CustomerAddress',
				'BookingState'
			)
		);

		return $paginate;
	}

/**
 * Admin index columns
 *
 * @return array
 */
	protected function _adminIndexColumns() {
		$Booking = $this->{$this->modelClass};

		$columns = parent::_adminIndexColumns();

		// Remove created/modified columns.
		unset($columns['Booking.created']);
		unset($columns['Booking.modified']);

		$columns[$Booking->alias . '.total_cost']['type'] = 'currency';

		$newColumns = array(
			'CustomerAddress.full_name' => array(
				'label' => __d('buzz_bookings', 'Customer'),
				'type' => 'string'
			),
			'BookingState.name' => array(
				'label' => __d('buzz_bookings', 'Status'),
				'type' => 'string'
			)
		);

		return ArrayUtil::addAfter($columns, 'Booking.sales_ref', $newColumns);
	}

/**
 * Admin columns whitelist
 *
 * @return array
 */
	protected function _adminIndexColumnsWhitelist() {
		$Booking = $this->{$this->modelClass};

		$whitelist = parent::_adminIndexColumnsWhitelist();
		$whitelist[] = $Booking->alias . '.sales_ref';
		$whitelist[] = $Booking->alias . '.total_cost';
		$whitelist[] = $Booking->alias . '.completed_date';

		return $whitelist;
	}

/**
 * Filters
 *
 * @return array
 */
	protected function _adminFilterFields() {
		$filters = parent::_adminFilterFields();

		unset($filters['Booking.name']);
		unset($filters['Booking.created']);
		unset($filters['Booking.modified']);

		$newFilters = array(
			'Booking.sales_ref' => array(
				'label' => __d('buzz_bookings', 'Sales Ref'),
				'type' => 'string',
				'compare' => array('Booking.sales_ref' => "%s")
			),
			'CustomerAddress.full_name' => array(
				'label' => __d('buzz_bookings', 'Customer'),
				'type' => 'string',
				'compare' => array('CONCAT(CustomerAddress.first_name, " ", CustomerAddress.last_name) LIKE' => "%%%s%%")
			),
			'Booking.booking_state_id' => array(
				'label' => 'Status',
				'type' => 'select',
				'default' => BookingState::COMPLETE,
				'compare' => array('Booking.booking_state_id' => '%s')
			),
			'Booking.completed_date' => array(
				'label' => __d('buzz_bookings', 'Completed Date'),
				'type' => 'date',
				'compare' => array('Booking.completed_date' => '%s')
			)
		);

		return ArrayUtil::addAfter($filters, 'Booking.sales_ref', $newFilters);
	}

/**
 * Used to populate form drop down selects
 *
 * @return void
 */
	protected function _adminPopulateLookups() {
		$this->set('bookingStates', $this->Booking->BookingState->find('list'));
		return;
	}

	public function admin_edit($id = null) {
		parent::admin_edit($id);

		// We're overriding the scaffolding template as we want to customise
		// the tabs.
		$this->view = 'BuzzBookings.admin_form';

		return;
	}

	protected function _adminIndexToolbar($id = null) {
		return [];
	}

	protected function _adminFormToolbar($id = null) {
		$actions = parent::_adminFormToolbar($id);
		unset($actions['Add New']);

		$actions['Resend Email'] = array(
			'url' => array('action' => 'email_confirmation', $id),
			'icon' => 'envelope'
		);

		return $actions;
	}

/**
 * Re-sends the confirmation email to the customer and redirects to the
 * admin edit form.
 *
 * @param int $id Purchase ID
 */
	public function admin_email_confirmation($id) {
		$Booking = $this->{$this->modelClass}->alias;

		$Event = new CakeEvent('Model.Booking.completed', $this, array(
			'id' => $id
		));
		$this->getEventManager()->dispatch($Event);

		$this->Session->setFlash(
			array(
				'title' => __d('buzz_bookings', '%s confirmation sent', [InflectorExt::humanize($this->$Booking->displayName)]),
				'description' => __d('buzz_bookings', 'The confirmation email has been resent to the customer!')
			),
			'flash_success'
		);

		return $this->redirect(['action' => 'edit', $id]);
	}

}
